declare type EventListener<EventType> = (event: EventType) => void;
declare type EventListenerRef = any;
declare type EventListenerBundle<EventTypes> = {
    readonly [EventName in keyof EventTypes]?: EventListener<EventTypes[EventName]>;
};
export declare class EventEmitter<EventTypes extends Record<string, any>> {
    #private;
    private _isEventListener;
    /**
     * Check if there are listeners for event name
     */
    hasListener<EventName extends keyof EventTypes>(eventName: EventName): boolean;
    /**
     * Emit an event
     * @param eventName Event name
     * @param value Event value that will be passed as an argument to listeners
     */
    protected emit<EventName extends keyof EventTypes>(eventName: EventName, value: EventTypes[EventName]): void;
    /**
     * Add multiple listeners for events
     * @param listenersBundle Map of events to listeners
     * @param ref Reference value that can be used to remove the
     * listeners
     */
    on(listenersBundle: EventListenerBundle<EventTypes>, ref?: EventListenerRef): void;
    /**
     * Add a listener to an event
     * @param eventName Event name
     * @param listener Listener function
     * @param ref Reference value that can be used to remove this listener,
     * if omitted it will be the listener itself
     */
    addListener<EventName extends keyof EventTypes>(eventName: EventName, listener: EventListener<EventTypes[EventName]>, ref?: EventListenerRef): void;
    /**
     * Remove multiple listeners for events
     * @param eventNames Array of event names
     * @param ref Reference value that map to listeners
     */
    off<EventName extends keyof EventTypes>(eventNames: EventName[], ref: EventListenerRef): void;
    /**
     * Remove a listener to an event
     * @param eventName Event name
     * @param ref Reference value that was used when adding the listener,
     * this can be the listener itself
     */
    removeListener<EventName extends keyof EventTypes>(eventName: EventName, ref: EventListenerRef): void;
}
export {};
//# sourceMappingURL=event-emitter.d.ts.map