"use strict";
var _GameEvents_bound, _GameEvents_features, _GameEvents_gameStatus, _GameEvents_state, _GameEvents_retries, _GameEvents_running, _GameEvents_startingPromise;
Object.defineProperty(exports, "__esModule", { value: true });
exports.GameEvents = void 0;
const tslib_1 = require("tslib");
const binder_1 = require("./binder");
const _1 = require("./");
class GameEvents extends _1.EventEmitter {
    constructor(features, gameStatus) {
        super();
        _GameEvents_bound.set(this, void 0);
        _GameEvents_features.set(this, void 0);
        _GameEvents_gameStatus.set(this, void 0);
        _GameEvents_state.set(this, void 0);
        _GameEvents_retries.set(this, void 0);
        _GameEvents_running.set(this, void 0);
        _GameEvents_startingPromise.set(this, void 0);
        tslib_1.__classPrivateFieldSet(this, _GameEvents_bound, binder_1.binder(this), "f");
        tslib_1.__classPrivateFieldSet(this, _GameEvents_features, features, "f");
        tslib_1.__classPrivateFieldSet(this, _GameEvents_gameStatus, gameStatus, "f");
        tslib_1.__classPrivateFieldSet(this, _GameEvents_state, {}, "f");
        tslib_1.__classPrivateFieldSet(this, _GameEvents_retries, 25, "f");
        tslib_1.__classPrivateFieldSet(this, _GameEvents_running, false, "f");
        tslib_1.__classPrivateFieldSet(this, _GameEvents_startingPromise, null, "f");
    }
    /** Copy of current state, both info updates and last event values */
    get state() {
        return _1.Utils.objectCopy(tslib_1.__classPrivateFieldGet(this, _GameEvents_state, "f"));
    }
    /**
     * Call overwolf.games.events.setRequiredFeatures and bind listeners
     * @see https://overwolf.github.io/docs/api/overwolf-games-events#setrequiredfeaturesfeatures-callback
     */
    async start() {
        if (tslib_1.__classPrivateFieldGet(this, _GameEvents_running, "f")) {
            return;
        }
        if (tslib_1.__classPrivateFieldGet(this, _GameEvents_startingPromise, "f")) {
            return tslib_1.__classPrivateFieldGet(this, _GameEvents_startingPromise, "f");
        }
        tslib_1.__classPrivateFieldSet(this, _GameEvents_startingPromise, this._start(), "f");
        await tslib_1.__classPrivateFieldGet(this, _GameEvents_startingPromise, "f");
        tslib_1.__classPrivateFieldSet(this, _GameEvents_startingPromise, null, "f");
    }
    async _start() {
        this._stop();
        const success = await this.setRequiredFeatures();
        if (success) {
            overwolf.games.events.getInfo(tslib_1.__classPrivateFieldGet(this, _GameEvents_bound, "f").onGotInfo);
            this.setListeners();
        }
    }
    /** Remove all listeners and clear the {@link state} */
    async stop() {
        if (!tslib_1.__classPrivateFieldGet(this, _GameEvents_running, "f")) {
            return;
        }
        if (tslib_1.__classPrivateFieldGet(this, _GameEvents_startingPromise, "f")) {
            await tslib_1.__classPrivateFieldGet(this, _GameEvents_startingPromise, "f");
            tslib_1.__classPrivateFieldSet(this, _GameEvents_startingPromise, null, "f");
        }
        this._stop();
        tslib_1.__classPrivateFieldSet(this, _GameEvents_running, false, "f");
    }
    _stop() {
        for (const key in tslib_1.__classPrivateFieldGet(this, _GameEvents_state, "f")) {
            if (tslib_1.__classPrivateFieldGet(this, _GameEvents_state, "f").hasOwnProperty(key)) {
                delete tslib_1.__classPrivateFieldGet(this, _GameEvents_state, "f")[key];
            }
        }
        this.removeListeners();
    }
    /** Remove all listeners */
    destroy() {
        this.removeListeners();
        tslib_1.__classPrivateFieldSet(this, _GameEvents_running, false, "f");
    }
    removeListeners() {
        overwolf.games.events.onError.removeListener(tslib_1.__classPrivateFieldGet(this, _GameEvents_bound, "f").onError);
        overwolf.games.events.onInfoUpdates2
            .removeListener(tslib_1.__classPrivateFieldGet(this, _GameEvents_bound, "f").onInfoUpdate);
        overwolf.games.events.onNewEvents.removeListener(tslib_1.__classPrivateFieldGet(this, _GameEvents_bound, "f").onNewEvent);
    }
    setListeners() {
        this.removeListeners();
        overwolf.games.events.onError.addListener(tslib_1.__classPrivateFieldGet(this, _GameEvents_bound, "f").onError);
        overwolf.games.events.onInfoUpdates2.addListener(tslib_1.__classPrivateFieldGet(this, _GameEvents_bound, "f").onInfoUpdate);
        overwolf.games.events.onNewEvents.addListener(tslib_1.__classPrivateFieldGet(this, _GameEvents_bound, "f").onNewEvent);
    }
    onError(err) {
        console.log('GameEvents: error:', err);
    }
    /** Wrapper for overwolf.games.events.getInfo */
    getInfo() {
        return new Promise((resolve, reject) => {
            overwolf.games.events.getInfo((data) => {
                if (data && data.success && data.res) {
                    resolve(data.res);
                }
                else {
                    reject(data);
                }
            });
        });
    }
    onGotInfo(data) {
        if (!data || !data.success || !data.res)
            return;
        const info = data.res;
        for (const category in info) {
            if (category === 'features')
                continue;
            for (const key in info[category]) {
                if (!info[category].hasOwnProperty(key))
                    continue;
                const path = `${category}.${key}`;
                let val = info[category][key];
                if (val !== undefined &&
                    (tslib_1.__classPrivateFieldGet(this, _GameEvents_state, "f")[path] === undefined || tslib_1.__classPrivateFieldGet(this, _GameEvents_state, "f")[path] !== val)) {
                    tslib_1.__classPrivateFieldGet(this, _GameEvents_state, "f")[path] = val;
                    try {
                        val = JSON.parse(val);
                    }
                    catch (e) { }
                    const e = { path, category, key, val };
                    this.emit(path, e);
                    this.emit(`${category}.*`, e);
                    this.emit('*', e);
                }
            }
        }
    }
    onInfoUpdate(data) {
        if (!data || !data.info)
            return;
        const info = data.info;
        for (const category in info) {
            if (!info.hasOwnProperty(category))
                continue;
            for (const key in info[category]) {
                if (!info[category].hasOwnProperty(key))
                    continue;
                const path = `${category}.${key}`;
                let val = info[category][key];
                if (val !== undefined &&
                    (tslib_1.__classPrivateFieldGet(this, _GameEvents_state, "f")[path] === undefined || tslib_1.__classPrivateFieldGet(this, _GameEvents_state, "f")[path] !== val)) {
                    tslib_1.__classPrivateFieldGet(this, _GameEvents_state, "f")[path] = val;
                    try {
                        val = JSON.parse(val);
                    }
                    catch (e) { }
                    const e = { path, category, key, val };
                    this.emit(path, e);
                    this.emit(`${category}.*`, e);
                    this.emit('*', e);
                }
            }
        }
    }
    onNewEvent(data) {
        if (!data.events || !data.events.length)
            return;
        for (const event of data.events) {
            const category = 'events', key = event.name, path = `${category}.${key}`;
            let val = event.data;
            if (val !== undefined &&
                (tslib_1.__classPrivateFieldGet(this, _GameEvents_state, "f")[path] === undefined || tslib_1.__classPrivateFieldGet(this, _GameEvents_state, "f")[path] !== val)) {
                tslib_1.__classPrivateFieldGet(this, _GameEvents_state, "f")[path] = val;
            }
            try {
                val = JSON.parse(val);
            }
            catch (e) { }
            const e = { path, category, key, val };
            this.emit(path, e);
            this.emit(`${category}.*`, e);
            this.emit('*', e);
        }
    }
    _setRequiredFeatures() {
        return new Promise(resolve => {
            overwolf.games.events.setRequiredFeatures(tslib_1.__classPrivateFieldGet(this, _GameEvents_features, "f"), resolve);
        });
    }
    async setRequiredFeatures() {
        let tries = 0, result = null;
        while (tries < tslib_1.__classPrivateFieldGet(this, _GameEvents_retries, "f") && tslib_1.__classPrivateFieldGet(this, _GameEvents_gameStatus, "f").isRunning) {
            result = await this._setRequiredFeatures();
            if (result.success) {
                console.log(..._1.Utils.L('setRequiredFeatures(): success:', result));
                return !!(result.supportedFeatures && result.supportedFeatures.length);
            }
            await _1.Utils.delay(2000);
            tries++;
        }
        console.log(`setRequiredFeatures(): failure after ${tries + 1} tries:`, result);
        return false;
    }
}
exports.GameEvents = GameEvents;
_GameEvents_bound = new WeakMap(), _GameEvents_features = new WeakMap(), _GameEvents_gameStatus = new WeakMap(), _GameEvents_state = new WeakMap(), _GameEvents_retries = new WeakMap(), _GameEvents_running = new WeakMap(), _GameEvents_startingPromise = new WeakMap();
//# sourceMappingURL=game-events.js.map