"use strict";
var _GameStatus_bound, _GameStatus_lastGameID, _GameStatus_started, _GameStatus_startPromise;
Object.defineProperty(exports, "__esModule", { value: true });
exports.GameStatus = void 0;
const tslib_1 = require("tslib");
const binder_1 = require("./binder");
const event_emitter_1 = require("./event-emitter");
class GameStatus extends event_emitter_1.EventEmitter {
    constructor() {
        super();
        _GameStatus_bound.set(this, void 0);
        _GameStatus_lastGameID.set(this, void 0);
        _GameStatus_started.set(this, void 0);
        _GameStatus_startPromise.set(this, void 0);
        tslib_1.__classPrivateFieldSet(this, _GameStatus_bound, binder_1.binder(this), "f");
        this.isInFocus = false;
        this.isRunning = false;
        this.gameInfo = null;
        tslib_1.__classPrivateFieldSet(this, _GameStatus_lastGameID, null, "f");
        tslib_1.__classPrivateFieldSet(this, _GameStatus_started, false, "f");
        tslib_1.__classPrivateFieldSet(this, _GameStatus_startPromise, this.start(), "f");
    }
    async start() {
        if (tslib_1.__classPrivateFieldGet(this, _GameStatus_started, "f")) {
            return;
        }
        if (tslib_1.__classPrivateFieldGet(this, _GameStatus_startPromise, "f")) {
            await tslib_1.__classPrivateFieldGet(this, _GameStatus_startPromise, "f");
            return;
        }
        const gameInfo = await new Promise(resolve => overwolf.games.getRunningGameInfo(resolve));
        this.setGameInfo(gameInfo);
        this.isInFocus = !!(gameInfo && gameInfo.isInFocus);
        this.isRunning = !!(gameInfo && gameInfo.isRunning);
        overwolf.games.onGameInfoUpdated
            .removeListener(tslib_1.__classPrivateFieldGet(this, _GameStatus_bound, "f").onGameInfoUpdated);
        overwolf.games.onGameInfoUpdated.addListener(tslib_1.__classPrivateFieldGet(this, _GameStatus_bound, "f").onGameInfoUpdated);
        tslib_1.__classPrivateFieldSet(this, _GameStatus_started, true, "f");
    }
    /** Remove all listeners */
    destroy() {
        overwolf.games.onGameInfoUpdated
            .removeListener(tslib_1.__classPrivateFieldGet(this, _GameStatus_bound, "f").onGameInfoUpdated);
    }
    onGameInfoUpdated(e) {
        const isInFocus = !!(e && e.gameInfo && e.gameInfo.isInFocus), isRunning = !!(e && e.gameInfo && e.gameInfo.isRunning), isInFocusChanged = (isInFocus !== this.isInFocus), isRunningChanged = (isRunning !== this.isRunning);
        if (e && e.gameInfo) {
            this.setGameInfo(e.gameInfo);
        }
        else {
            this.setGameInfo(null);
        }
        if (isInFocusChanged) {
            this.isInFocus = isInFocus;
        }
        if (isRunningChanged) {
            this.isRunning = isRunning;
        }
        if (isInFocusChanged) {
            this.emit('focus', isInFocus);
        }
        if (isRunningChanged) {
            this.emit('running', isInFocus);
        }
        if (e && e.gameInfo && e.resolutionChanged) {
            this.emit('resolutionChanged', e.gameInfo);
        }
        if (e.gameChanged) {
            this.emit('gameChanged', e.gameInfo || null);
        }
        if (isInFocusChanged ||
            isRunningChanged ||
            e.resolutionChanged ||
            e.gameChanged) {
            this.emit('changed', e.gameInfo || null);
        }
    }
    setGameInfo(gameInfo) {
        if (gameInfo && gameInfo.isRunning) {
            this.gameInfo = gameInfo;
            tslib_1.__classPrivateFieldSet(this, _GameStatus_lastGameID, Math.floor(gameInfo.id / 10), "f");
        }
        else {
            this.gameInfo = null;
        }
    }
    /**
     * Check whenter the game running matches the passed GameID
     * @param id Overwolf GameID
     */
    gameIs(id) {
        return (this.isRunning && this.gameID === id);
    }
    /** Get gameID of current or last running game */
    get gameID() {
        if (this.gameInfo) {
            return Math.floor(this.gameInfo.id / 10);
        }
        return tslib_1.__classPrivateFieldGet(this, _GameStatus_lastGameID, "f");
    }
}
exports.GameStatus = GameStatus;
_GameStatus_bound = new WeakMap(), _GameStatus_lastGameID = new WeakMap(), _GameStatus_started = new WeakMap(), _GameStatus_startPromise = new WeakMap();
//# sourceMappingURL=game-status.js.map