"use strict";
var _HotkeyService_bound, _HotkeyService_hotkeys, _HotkeyService_started, _HotkeyService_startPromise;
Object.defineProperty(exports, "__esModule", { value: true });
exports.HotkeyService = void 0;
const tslib_1 = require("tslib");
const binder_1 = require("./binder");
const event_emitter_1 = require("./event-emitter");
class HotkeyService extends event_emitter_1.EventEmitter {
    constructor() {
        super();
        _HotkeyService_bound.set(this, binder_1.binder(this));
        _HotkeyService_hotkeys.set(this, void 0);
        _HotkeyService_started.set(this, void 0);
        _HotkeyService_startPromise.set(this, void 0);
        tslib_1.__classPrivateFieldSet(this, _HotkeyService_bound, binder_1.binder(this), "f");
        tslib_1.__classPrivateFieldSet(this, _HotkeyService_hotkeys, {}, "f");
        tslib_1.__classPrivateFieldSet(this, _HotkeyService_started, false, "f");
        tslib_1.__classPrivateFieldSet(this, _HotkeyService_startPromise, this.start(), "f");
    }
    async start() {
        if (tslib_1.__classPrivateFieldGet(this, _HotkeyService_started, "f")) {
            return;
        }
        if (tslib_1.__classPrivateFieldGet(this, _HotkeyService_startPromise, "f")) {
            await tslib_1.__classPrivateFieldGet(this, _HotkeyService_startPromise, "f");
            return;
        }
        overwolf.settings.hotkeys.onChanged
            .removeListener(tslib_1.__classPrivateFieldGet(this, _HotkeyService_bound, "f").handleHotkeyChanged);
        overwolf.settings.hotkeys.onPressed
            .removeListener(tslib_1.__classPrivateFieldGet(this, _HotkeyService_bound, "f").handleHotkeyPressed);
        overwolf.settings.hotkeys.onChanged
            .addListener(tslib_1.__classPrivateFieldGet(this, _HotkeyService_bound, "f").handleHotkeyChanged);
        overwolf.settings.hotkeys.onPressed
            .addListener(tslib_1.__classPrivateFieldGet(this, _HotkeyService_bound, "f").handleHotkeyPressed);
        await this.updateHotkeys();
        for (const name in tslib_1.__classPrivateFieldGet(this, _HotkeyService_hotkeys, "f")) {
            if (tslib_1.__classPrivateFieldGet(this, _HotkeyService_hotkeys, "f").hasOwnProperty(name)) {
                this.emit('changed', {
                    name,
                    binding: tslib_1.__classPrivateFieldGet(this, _HotkeyService_hotkeys, "f")[name].global
                });
                for (const gameIdStr in tslib_1.__classPrivateFieldGet(this, _HotkeyService_hotkeys, "f")[name].games) {
                    if (tslib_1.__classPrivateFieldGet(this, _HotkeyService_hotkeys, "f")[name].games.hasOwnProperty(name)) {
                        const gameId = parseInt(gameIdStr, 10);
                        this.emit('changed', {
                            name,
                            gameId,
                            binding: tslib_1.__classPrivateFieldGet(this, _HotkeyService_hotkeys, "f")[name].games[gameIdStr]
                        });
                    }
                }
            }
        }
        tslib_1.__classPrivateFieldSet(this, _HotkeyService_started, true, "f");
    }
    /** Remove all listeners */
    destroy() {
        overwolf.settings.hotkeys.onChanged
            .removeListener(tslib_1.__classPrivateFieldGet(this, _HotkeyService_bound, "f").handleHotkeyChanged);
        overwolf.settings.hotkeys.onPressed
            .removeListener(tslib_1.__classPrivateFieldGet(this, _HotkeyService_bound, "f").handleHotkeyPressed);
    }
    getHotkeyBinding(hotkeyName, gameId) {
        if (tslib_1.__classPrivateFieldGet(this, _HotkeyService_hotkeys, "f")[hotkeyName]) {
            if (typeof gameId === 'number' &&
                tslib_1.__classPrivateFieldGet(this, _HotkeyService_hotkeys, "f")[hotkeyName].games &&
                tslib_1.__classPrivateFieldGet(this, _HotkeyService_hotkeys, "f")[hotkeyName].games[gameId]) {
                return tslib_1.__classPrivateFieldGet(this, _HotkeyService_hotkeys, "f")[hotkeyName].games[gameId];
            }
            if (tslib_1.__classPrivateFieldGet(this, _HotkeyService_hotkeys, "f")[hotkeyName].global) {
                return tslib_1.__classPrivateFieldGet(this, _HotkeyService_hotkeys, "f")[hotkeyName].global;
            }
        }
        return null;
    }
    getHotkeys() {
        return new Promise((resolve, reject) => {
            overwolf.settings.hotkeys.get(results => {
                if (results.success) {
                    resolve(results);
                }
                else {
                    reject(new Error(results.error));
                }
            });
        });
    }
    async updateHotkeys() {
        const hotkeysResult = await this.getHotkeys();
        for (const hotkey of hotkeysResult.globals) {
            if (!tslib_1.__classPrivateFieldGet(this, _HotkeyService_hotkeys, "f")[hotkey.name]) {
                tslib_1.__classPrivateFieldGet(this, _HotkeyService_hotkeys, "f")[hotkey.name] = {
                    global: hotkey.binding,
                    games: {}
                };
            }
            else {
                tslib_1.__classPrivateFieldGet(this, _HotkeyService_hotkeys, "f")[hotkey.name].global = hotkey.binding;
            }
        }
        if (hotkeysResult.games !== undefined) {
            for (const gameId in hotkeysResult.games) {
                if (hotkeysResult.games.hasOwnProperty(gameId)) {
                    for (const hotkey of hotkeysResult.games[gameId]) {
                        if (!tslib_1.__classPrivateFieldGet(this, _HotkeyService_hotkeys, "f")[hotkey.name]) {
                            tslib_1.__classPrivateFieldGet(this, _HotkeyService_hotkeys, "f")[hotkey.name] = {
                                global: '',
                                games: {}
                            };
                        }
                        if (!tslib_1.__classPrivateFieldGet(this, _HotkeyService_hotkeys, "f")[hotkey.name].games) {
                            tslib_1.__classPrivateFieldGet(this, _HotkeyService_hotkeys, "f")[hotkey.name].games = {};
                        }
                        tslib_1.__classPrivateFieldGet(this, _HotkeyService_hotkeys, "f")[hotkey.name].games[gameId] = hotkey.binding;
                    }
                }
            }
        }
    }
    handleHotkeyChanged(event) {
        console.log('HotkeyService.handleHotkeyChanged():', event);
        if (event.gameId === 0) {
            tslib_1.__classPrivateFieldGet(this, _HotkeyService_hotkeys, "f")[event.name].global = event.binding;
        }
        else if (tslib_1.__classPrivateFieldGet(this, _HotkeyService_hotkeys, "f")[event.name].games !== undefined) {
            tslib_1.__classPrivateFieldGet(this, _HotkeyService_hotkeys, "f")[event.name].games[event.gameId] = event.binding;
        }
        const publicEvent = {
            name: event.name,
            binding: event.binding
        };
        if (event.gameId !== 0) {
            publicEvent.gameId = event.gameId;
        }
        this.emit('changed', publicEvent);
    }
    handleHotkeyPressed(event) {
        this.emit('pressed', event.name);
    }
    /**
     * Wrapper for overwolf.settings.hotkeys.assign
     * @see https://overwolf.github.io/docs/api/overwolf-settings-hotkeys#assignhotkey-callback
     */
    assignHotkey(assignHotkey) {
        return new Promise(resolve => {
            overwolf.settings.hotkeys.assign(assignHotkey, resolve);
        });
    }
    /**
     * Wrapper for overwolf.settings.hotkeys.unassign
     * @see https://overwolf.github.io/docs/api/overwolf-settings-hotkeys#unassignhotkey-callback
     */
    unassignHotkey(unassignHotkey) {
        return new Promise(resolve => {
            overwolf.settings.hotkeys.unassign(unassignHotkey, resolve);
        });
    }
}
exports.HotkeyService = HotkeyService;
_HotkeyService_bound = new WeakMap(), _HotkeyService_hotkeys = new WeakMap(), _HotkeyService_started = new WeakMap(), _HotkeyService_startPromise = new WeakMap();
//# sourceMappingURL=hotkeys-service.js.map