"use strict";
var _LauncherStatus_bound, _LauncherStatus_isInFocus, _LauncherStatus_launcherInfo, _LauncherStatus_started, _LauncherStatus_startPromise;
Object.defineProperty(exports, "__esModule", { value: true });
exports.LauncherStatus = void 0;
const tslib_1 = require("tslib");
const binder_1 = require("./binder");
const event_emitter_1 = require("./event-emitter");
const utils_1 = require("./utils");
class LauncherStatus extends event_emitter_1.EventEmitter {
    constructor() {
        super();
        _LauncherStatus_bound.set(this, void 0);
        _LauncherStatus_isInFocus.set(this, void 0);
        _LauncherStatus_launcherInfo.set(this, void 0);
        _LauncherStatus_started.set(this, void 0);
        _LauncherStatus_startPromise.set(this, void 0);
        tslib_1.__classPrivateFieldSet(this, _LauncherStatus_bound, binder_1.binder(this), "f");
        tslib_1.__classPrivateFieldSet(this, _LauncherStatus_launcherInfo, null, "f");
        tslib_1.__classPrivateFieldSet(this, _LauncherStatus_isInFocus, false, "f");
        tslib_1.__classPrivateFieldSet(this, _LauncherStatus_started, false, "f");
        tslib_1.__classPrivateFieldSet(this, _LauncherStatus_startPromise, this.start(), "f");
    }
    get isRunning() {
        return !!tslib_1.__classPrivateFieldGet(this, _LauncherStatus_launcherInfo, "f");
    }
    get isInFocus() {
        return tslib_1.__classPrivateFieldGet(this, _LauncherStatus_isInFocus, "f");
    }
    get launcherInfo() {
        return utils_1.objectCopy(tslib_1.__classPrivateFieldGet(this, _LauncherStatus_launcherInfo, "f"));
    }
    async start() {
        if (tslib_1.__classPrivateFieldGet(this, _LauncherStatus_started, "f")) {
            return;
        }
        if (tslib_1.__classPrivateFieldGet(this, _LauncherStatus_startPromise, "f")) {
            await tslib_1.__classPrivateFieldGet(this, _LauncherStatus_startPromise, "f");
            return;
        }
        const launchersInfo = await new Promise(resolve => {
            overwolf.games.launchers.getRunningLaunchersInfo(resolve);
        });
        if (launchersInfo.success &&
            launchersInfo.launchers &&
            launchersInfo.launchers.length) {
            tslib_1.__classPrivateFieldSet(this, _LauncherStatus_launcherInfo, launchersInfo.launchers[0], "f");
        }
        else {
            tslib_1.__classPrivateFieldSet(this, _LauncherStatus_launcherInfo, null, "f");
        }
        tslib_1.__classPrivateFieldSet(this, _LauncherStatus_isInFocus, !!(tslib_1.__classPrivateFieldGet(this, _LauncherStatus_launcherInfo, "f") && tslib_1.__classPrivateFieldGet(this, _LauncherStatus_launcherInfo, "f").isInFocus), "f");
        overwolf.games.launchers.onLaunched
            .removeListener(tslib_1.__classPrivateFieldGet(this, _LauncherStatus_bound, "f").onLauncherLaunched);
        overwolf.games.launchers.onTerminated
            .removeListener(tslib_1.__classPrivateFieldGet(this, _LauncherStatus_bound, "f").onLauncherTerminated);
        overwolf.games.launchers.onUpdated
            .removeListener(tslib_1.__classPrivateFieldGet(this, _LauncherStatus_bound, "f").onLauncherInfoUpdated);
        overwolf.games.launchers.onLaunched
            .addListener(tslib_1.__classPrivateFieldGet(this, _LauncherStatus_bound, "f").onLauncherLaunched);
        overwolf.games.launchers.onTerminated
            .addListener(tslib_1.__classPrivateFieldGet(this, _LauncherStatus_bound, "f").onLauncherTerminated);
        overwolf.games.launchers.onUpdated
            .addListener(tslib_1.__classPrivateFieldGet(this, _LauncherStatus_bound, "f").onLauncherInfoUpdated);
        tslib_1.__classPrivateFieldSet(this, _LauncherStatus_started, true, "f");
    }
    /** Remove all listeners */
    destroy() {
        overwolf.games.launchers.onLaunched
            .removeListener(tslib_1.__classPrivateFieldGet(this, _LauncherStatus_bound, "f").onLauncherLaunched);
        overwolf.games.launchers.onTerminated
            .removeListener(tslib_1.__classPrivateFieldGet(this, _LauncherStatus_bound, "f").onLauncherTerminated);
        overwolf.games.launchers.onUpdated
            .removeListener(tslib_1.__classPrivateFieldGet(this, _LauncherStatus_bound, "f").onLauncherInfoUpdated);
    }
    onLauncherLaunched(info) {
        // console.log('onLauncherLaunched', info)
        const isInFocusChanged = (info.isInFocus !== tslib_1.__classPrivateFieldGet(this, _LauncherStatus_isInFocus, "f"));
        tslib_1.__classPrivateFieldSet(this, _LauncherStatus_launcherInfo, info, "f");
        if (isInFocusChanged) {
            tslib_1.__classPrivateFieldSet(this, _LauncherStatus_isInFocus, info.isInFocus, "f");
            this.emit('focus', info.isInFocus);
        }
        this.emit('running', true);
        this.emit('changed', this.launcherInfo);
    }
    onLauncherTerminated() {
        // console.log('onLauncherTerminated')
        const isInFocusChanged = tslib_1.__classPrivateFieldGet(this, _LauncherStatus_isInFocus, "f");
        tslib_1.__classPrivateFieldSet(this, _LauncherStatus_launcherInfo, null, "f");
        tslib_1.__classPrivateFieldSet(this, _LauncherStatus_isInFocus, false, "f");
        if (isInFocusChanged) {
            this.emit('focus', false);
        }
        this.emit('running', false);
        this.emit('changed', this.launcherInfo);
    }
    onLauncherInfoUpdated({ info }) {
        // console.log('onLauncherInfoUpdated', info, changeType)
        const isInFocusChanged = (info.isInFocus !== tslib_1.__classPrivateFieldGet(this, _LauncherStatus_isInFocus, "f"));
        tslib_1.__classPrivateFieldSet(this, _LauncherStatus_launcherInfo, info, "f");
        if (isInFocusChanged) {
            tslib_1.__classPrivateFieldSet(this, _LauncherStatus_isInFocus, info.isInFocus, "f");
            this.emit('focus', info.isInFocus);
        }
        this.emit('changed', this.launcherInfo);
    }
    get launcherID() {
        if (tslib_1.__classPrivateFieldGet(this, _LauncherStatus_launcherInfo, "f") && tslib_1.__classPrivateFieldGet(this, _LauncherStatus_launcherInfo, "f").classId) {
            return tslib_1.__classPrivateFieldGet(this, _LauncherStatus_launcherInfo, "f").classId;
        }
        return null;
    }
}
exports.LauncherStatus = LauncherStatus;
_LauncherStatus_bound = new WeakMap(), _LauncherStatus_isInFocus = new WeakMap(), _LauncherStatus_launcherInfo = new WeakMap(), _LauncherStatus_started = new WeakMap(), _LauncherStatus_startPromise = new WeakMap();
//# sourceMappingURL=launcher-status.js.map