"use strict";
var _StateManager_state, _StateManager_persistent, _StateManager_localStoragePrefix;
Object.defineProperty(exports, "__esModule", { value: true });
exports.makeStateClient = exports.makeState = void 0;
const tslib_1 = require("tslib");
const _1 = require("./");
class StateManager extends _1.EventEmitter {
    constructor(name, initialState, persistent = false) {
        super();
        // readonly #initialState: StateMap
        _StateManager_state.set(this, void 0);
        _StateManager_persistent.set(this, void 0);
        _StateManager_localStoragePrefix.set(this, void 0);
        this.name = name;
        tslib_1.__classPrivateFieldSet(this, _StateManager_persistent, persistent, "f");
        tslib_1.__classPrivateFieldSet(this, _StateManager_localStoragePrefix, `${name}/`, "f");
        // this.#initialState = initialState;
        if (tslib_1.__classPrivateFieldGet(this, _StateManager_persistent, "f")) {
            tslib_1.__classPrivateFieldSet(this, _StateManager_state, StateManager.hydratePersistentState(initialState, tslib_1.__classPrivateFieldGet(this, _StateManager_localStoragePrefix, "f")), "f");
        }
        else {
            tslib_1.__classPrivateFieldSet(this, _StateManager_state, _1.Utils.objectCopy(initialState), "f");
        }
    }
    has(key) {
        return (tslib_1.__classPrivateFieldGet(this, _StateManager_state, "f")[key] !== undefined);
    }
    get(key) {
        const value = tslib_1.__classPrivateFieldGet(this, _StateManager_state, "f")[key];
        if (typeof value === 'object' && value !== null) {
            return _1.Utils.objectCopy(value);
        }
        return value;
    }
    _setObject(key, value) {
        const stringified = JSON.stringify(value);
        if (tslib_1.__classPrivateFieldGet(this, _StateManager_persistent, "f")) {
            localStorage.setItem(tslib_1.__classPrivateFieldGet(this, _StateManager_localStoragePrefix, "f") + key, stringified);
        }
        const copy = JSON.parse(stringified);
        tslib_1.__classPrivateFieldGet(this, _StateManager_state, "f")[key] = copy;
        this.emit(key, copy);
    }
    _setPrimitive(key, value) {
        if (tslib_1.__classPrivateFieldGet(this, _StateManager_persistent, "f")) {
            localStorage.setItem(tslib_1.__classPrivateFieldGet(this, _StateManager_localStoragePrefix, "f") + key, JSON.stringify(value));
        }
        tslib_1.__classPrivateFieldGet(this, _StateManager_state, "f")[key] = value;
        this.emit(key, value);
    }
    set(key, value) {
        if (typeof value === 'object' && value !== null) {
            this._setObject(key, value);
        }
        else {
            this._setPrimitive(key, value);
        }
    }
    replaceState(state) {
        tslib_1.__classPrivateFieldSet(this, _StateManager_state, _1.Utils.objectCopy(state), "f");
    }
    clearPersistentState() {
        for (const key in tslib_1.__classPrivateFieldGet(this, _StateManager_state, "f")) {
            if (tslib_1.__classPrivateFieldGet(this, _StateManager_state, "f").hasOwnProperty(key)) {
                localStorage.removeItem(tslib_1.__classPrivateFieldGet(this, _StateManager_localStoragePrefix, "f") + key);
            }
        }
    }
    static hydratePersistentState(initialState, prefix) {
        const state = _1.Utils.objectCopy(initialState), prefixLength = prefix.length;
        for (const key in localStorage) {
            if (key.length > prefixLength && key.startsWith(prefix)) {
                const stateKey = key.substr(prefixLength);
                const stored = localStorage.getItem(key);
                if (typeof stored === 'string' && stored !== 'undefined') {
                    state[stateKey] = JSON.parse(stored);
                }
            }
        }
        return state;
    }
}
_StateManager_state = new WeakMap(), _StateManager_persistent = new WeakMap(), _StateManager_localStoragePrefix = new WeakMap();
function makeGlobalStateName(name) {
    return `GlobalState/${name}`;
}
function makeState(name, initialState, persistent = false) {
    const manager = new StateManager(name, initialState, persistent);
    const store = new Proxy(manager, {
        has(target, key) {
            return target.has(key);
        },
        get(target, key) {
            switch (key) {
                case 'on':
                    return target.on.bind(target);
                case 'off':
                    return target.off.bind(target);
                case 'addListener':
                    return target.addListener.bind(target);
                case 'removeListener':
                    return target.removeListener.bind(target);
                default:
                    return target.get(key);
            }
        },
        set(target, key, value) {
            switch (key) {
                case 'on':
                case 'off':
                case 'addListener':
                case 'removeListener':
                    throw new Error(`${key} is a reserved property for Store`);
            }
            target.set(key, value);
            return true;
        },
        deleteProperty() {
            return false;
        },
        ownKeys() {
            return [];
        }
    });
    _1.WindowTunnel.set(makeGlobalStateName(name), store);
    return store;
}
exports.makeState = makeState;
function makeStateClient(name) {
    return _1.WindowTunnel.get(makeGlobalStateName(name));
}
exports.makeStateClient = makeStateClient;
//# sourceMappingURL=store.js.map