"use strict";
Object.defineProperty(exports, "__esModule", { value: true });
exports.beforeClose = exports.log = exports.L = exports.capitalizeFirstLetter = exports.formatNumber = exports.arrayChunk = exports.objectCopy = exports.delay = exports.toPrettyJSON = void 0;
function toPrettyJSON(input) {
    return JSON.stringify(input, null, '    ');
}
exports.toPrettyJSON = toPrettyJSON;
async function delay(time) {
    await new Promise(resolve => setTimeout(resolve, time));
}
exports.delay = delay;
function objectCopy(input) {
    return JSON.parse(JSON.stringify(input));
}
exports.objectCopy = objectCopy;
function arrayChunk(arr, len) {
    const n = arr.length, chunks = [];
    let i = 0;
    while (i < n) {
        chunks.push(arr.slice(i, i += len));
    }
    return chunks;
}
exports.arrayChunk = arrayChunk;
function formatNumber(val) {
    if (val > 9999)
        return val.toLocaleString('en-US');
    return String(val);
}
exports.formatNumber = formatNumber;
function capitalizeFirstLetter(string) {
    return string.charAt(0).toUpperCase() + string.slice(1);
}
exports.capitalizeFirstLetter = capitalizeFirstLetter;
function L(...args) {
    const len = args.length, out = [];
    for (let i = 0; i < len; i++) {
        const val = args[i];
        if (typeof val === 'object' && !(val instanceof Error)) {
            out.push(val, `${JSON.stringify(val)}`);
        }
        else {
            out.push(val);
        }
    }
    return out;
}
exports.L = L;
exports.log = L;
/**
 * Set a callback that will be fired before the current window closes
 * @param fn Callback that will be fired before the current window closes
 */
function beforeClose(fn) {
    window.addEventListener('beforeunload', e => {
        console.log('beforeunload fired', e);
        delete e.returnValue;
        fn();
        console.log('beforeunload complete');
    });
}
exports.beforeClose = beforeClose;
//# sourceMappingURL=utils.js.map