"use strict";
Object.defineProperty(exports, "__esModule", { value: true });
exports.WindowTunnel = void 0;
function makeGlobalKey(key) {
    return `WindowTunnel/${key}`;
}
exports.WindowTunnel = {
    /**
     * Set a global object that will be shared between all windows.
     * This should ONLY be called from the window set as "start_window"
     * in manifest.json (background window)
     * @param key Global name for tunneled object
     * @param value Object to tunnel
     */
    set(key, value) {
        // console.log('WindowTunnel.set()', key, value);
        const w = window;
        w[makeGlobalKey(key)] = value;
    },
    /**
     * Get a global tunneled object.
     * @param key Global name for tunneled object
     * @returns Tunneled object
     */
    get(key) {
        const w = overwolf.windows.getMainWindow();
        const globalKey = makeGlobalKey(key);
        if (w[globalKey] === undefined) {
            throw new Error(`WindowTunnel.get(): "${key}" failed`);
        }
        // console.log('WindowTunnel.get()', key, w[globalKey]);
        return w[globalKey];
    }
};
//# sourceMappingURL=window-tunnel.js.map